@@ -1,43 +1,42 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-    make-release
-    ~~~~~~~~~~~~
+from __future__ import print_function
 
-    Helper script that performs a release.  Does pretty much everything
-    automatically for us.
-
-    :copyright: (c) 2014 by Armin Ronacher.
-    :license: BSD, see LICENSE for more details.
-"""
-import sys
-import shutil
 import os
 import re
-from datetime import datetime, date
-from subprocess import Popen, PIPE
+import sys
+from datetime import date, datetime
+from subprocess import PIPE, Popen
 
-_date_clean_re = re.compile(r'(\d+)(st|nd|rd|th)')
+_date_strip_re = re.compile(r'(?<=\d)(st|nd|rd|th)')
 
 
 def parse_changelog():
-    with open('CHANGES') as f:
+    with open('CHANGES.rst') as f:
         lineiter = iter(f)
         for line in lineiter:
             match = re.search('^Version\s+(.*)', line.strip())
+
             if match is None:
                 continue
+
             version = match.group(1).strip()
-            if lineiter.next().count('-') != len(match.group(0)):
+
+            if next(lineiter).count('-') != len(match.group(0)):
                 continue
+
             while 1:
-                change_info = lineiter.next().strip()
+                change_info = next(lineiter).strip()
+
                 if change_info:
                     break
 
-            match = re.search(r'released on (\w+\s+\d+\w+\s+\d+)'
-                              r'(?:, codename (.*))?', change_info,
-                              flags=re.IGNORECASE)
+            match = re.search(
+                r'released on (\w+\s+\d+\w+\s+\d+)(?:, codename (.*))?',
+                change_info,
+                flags=re.IGNORECASE
+            )
+
             if match is None:
                 continue
 
@@ -47,15 +46,16 @@ def parse_changelog():
 
 def bump_version(version):
     try:
-        parts = map(int, version.split('.'))
+        parts = [int(i) for i in version.split('.')]
     except ValueError:
         fail('Current version is not numeric')
+
     parts[-1] += 1
     return '.'.join(map(str, parts))
 
 
 def parse_date(string):
-    string = _date_clean_re.sub(r'\1', string)
+    string = _date_strip_re.sub('', string)
     return datetime.strptime(string, '%B %d %Y')
 
 
@@ -66,10 +66,13 @@ def set_filename_version(filename, version_number, pattern):
         before, old, after = match.groups()
         changed.append(True)
         return before + version_number + after
+
     with open(filename) as f:
-        contents = re.sub(r"^(\s*%s\s*=\s*')(.+?)(')" % pattern,
-                          inject_version, f.read(),
-                          flags=re.DOTALL | re.MULTILINE)
+        contents = re.sub(
+            r"^(\s*%s\s*=\s*')(.+?)(')" % pattern,
+            inject_version, f.read(),
+            flags=re.DOTALL | re.MULTILINE
+        )
 
     if not changed:
         fail('Could not find %s in %s', pattern, filename)
@@ -83,28 +86,24 @@ def set_init_version(version):
     set_filename_version('werkzeug/__init__.py', version, '__version__')
 
 
-def build_and_upload():
-    # Work around setuptools packaging stray .pyc files
-    if os.path.exists('werkzeug/testsuite'):
-        shutil.rmtree('werkzeug/testsuite')
-
-    Popen([sys.executable, 'setup.py', 'release', 'sdist', 'bdist_wheel',
-           'upload']).wait()
+def build():
+    cmd = [sys.executable, 'setup.py', 'sdist', 'bdist_wheel']
+    Popen(cmd).wait()
 
 
 def fail(message, *args):
-    print >> sys.stderr, 'Error:', message % args
+    print('Error:', message % args, file=sys.stderr)
     sys.exit(1)
 
 
 def info(message, *args):
-    print >> sys.stderr, message % args
+    print(message % args, file=sys.stderr)
 
 
 def get_git_tags():
-    return set(Popen(['git', 'tag'], stdout=PIPE)
-               .communicate()[0]
-               .splitlines())
+    return set(
+        Popen(['git', 'tag'], stdout=PIPE).communicate()[0].splitlines()
+    )
 
 
 def git_is_clean():
@@ -125,29 +124,40 @@ def main():
     os.chdir(os.path.join(os.path.dirname(__file__), '..'))
 
     rv = parse_changelog()
+
     if rv is None:
         fail('Could not parse changelog')
 
     version, release_date, codename = rv
-    dev_version = bump_version(version) + '-dev'
+    dev_version = bump_version(version) + '.dev'
 
-    info('Releasing %s (codename %s, release date %s)',
-         version, codename, release_date.strftime('%d/%m/%Y'))
+    info(
+        'Releasing %s (codename %s, release date %s)',
+        version, codename, release_date.strftime('%d/%m/%Y')
+    )
     tags = get_git_tags()
 
     if version in tags:
         fail('Version "%s" is already tagged', version)
+
     if release_date.date() != date.today():
-        fail('Release date is not today (%s != %s)',
-             release_date.date(), date.today())
+        fail(
+            'Release date is not today (%s != %s)',
+            release_date.date(), date.today()
+        )
 
     if not git_is_clean():
         fail('You have uncommitted changes in git')
 
+    try:
+        import wheel  # noqa: F401
+    except ImportError:
+        fail('You need to install the wheel package.')
+
     set_init_version(version)
     make_git_commit('Bump version number to %s', version)
     make_git_tag(version)
-    build_and_upload()
+    build()
     set_init_version(dev_version)
 
 
