@@ -395,6 +395,38 @@ class TestMultiDict(_MutableMultiDictTests):
         assert list(zip(iterkeys(md), iterlistvalues(md))) == \
             list(iterlists(md))
 
+    @pytest.mark.skipif(not PY2, reason='viewmethods work only for the 2-nd version.')
+    def test_view_methods(self):
+        mapping = [('a', 'b'), ('a', 'c')]
+        md = self.storage_class(mapping)
+
+        vi = md.viewitems()
+        vk = md.viewkeys()
+        vv = md.viewvalues()
+
+        assert list(vi) == list(md.items())
+        assert list(vk) == list(md.keys())
+        assert list(vv) == list(md.values())
+
+        md['k'] = 'n'
+
+        assert list(vi) == list(md.items())
+        assert list(vk) == list(md.keys())
+        assert list(vv) == list(md.values())
+
+    @pytest.mark.skipif(not PY2, reason='viewmethods work only for the 2-nd version.')
+    def test_viewitems_with_multi(self):
+        mapping = [('a', 'b'), ('a', 'c')]
+        md = self.storage_class(mapping)
+
+        vi = md.viewitems(multi=True)
+
+        assert list(vi) == list(md.items(multi=True))
+
+        md['k'] = 'n'
+
+        assert list(vi) == list(md.items(multi=True))
+
     def test_getitem_raise_badrequestkeyerror_for_empty_list_value(self):
         mapping = [('a', 'b'), ('a', 'c')]
         md = self.storage_class(mapping)
@@ -494,6 +526,11 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         with pytest.raises(BadRequestKeyError):
             d.popitemlist()
 
+        # Unhashable
+        d = self.storage_class()
+        d.add('foo', 23)
+        pytest.raises(TypeError, hash, d)
+
     def test_iterables(self):
         a = datastructures.MultiDict((("key_a", "value_a"),))
         b = datastructures.MultiDict((("key_b", "value_b"),))
@@ -508,6 +545,24 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         assert sorted(iterkeys(ab)) == ["key_a", "key_b"]
 
 
+class TestTypeConversionDict(object):
+    storage_class = datastructures.TypeConversionDict
+
+    def test_value_conversion(self):
+        d = self.storage_class(foo='1')
+        assert d.get('foo', type=int) == 1
+
+    def test_return_default_when_conversion_is_not_possible(self):
+        d = self.storage_class(foo='bar')
+        assert d.get('foo', default=-1, type=int) == -1
+
+    def test_propagate_exceptions_in_conversion(self):
+        d = self.storage_class(foo='bar')
+        switch = {'a': 1}
+        with pytest.raises(KeyError):
+            d.get('foo', type=lambda x: switch[x])
+
+
 class TestCombinedMultiDict(object):
     storage_class = datastructures.CombinedMultiDict
 
@@ -716,6 +771,24 @@ class TestEnvironHeaders(object):
         ]
         assert not self.storage_class({'wsgi.version': (1, 0)})
         assert len(self.storage_class({'wsgi.version': (1, 0)})) == 0
+        assert 42 not in headers
+
+    def test_skip_empty_special_vars(self):
+        env = {
+            'HTTP_X_FOO':               '42',
+            'CONTENT_TYPE':             '',
+            'CONTENT_LENGTH':           '',
+        }
+        headers = self.storage_class(env)
+        assert dict(headers) == {'X-Foo': '42'}
+
+        env = {
+            'HTTP_X_FOO':               '42',
+            'CONTENT_TYPE':             '',
+            'CONTENT_LENGTH':           '0',
+        }
+        headers = self.storage_class(env)
+        assert dict(headers) == {'X-Foo': '42', 'Content-Length': '0'}
 
     def test_return_type_is_unicode(self):
         # environ contains native strings; we return unicode
@@ -770,10 +843,10 @@ class TestImmutableList(object):
     storage_class = datastructures.ImmutableList
 
     def test_list_hashable(self):
-        t = (1, 2, 3, 4)
-        l = self.storage_class(t)
-        assert hash(t) == hash(l)
-        assert t != l
+        data = (1, 2, 3, 4)
+        store = self.storage_class(data)
+        assert hash(data) == hash(store)
+        assert data != store
 
 
 def make_call_asserter(func=None):
@@ -911,7 +984,14 @@ class TestAccept(object):
             'asterisk'
         assert accept.best_match(['star'], default=None) is None
 
-    @pytest.mark.skipif(True, reason='Werkzeug doesn\'t respect specificity.')
+    def test_accept_keep_order(self):
+        accept = self.storage_class([('*', 1)])
+        assert accept.best_match(["alice", "bob"]) == "alice"
+        assert accept.best_match(["bob", "alice"]) == "bob"
+        accept = self.storage_class([('alice', 1), ('bob', 1)])
+        assert accept.best_match(["alice", "bob"]) == "alice"
+        assert accept.best_match(["bob", "alice"]) == "bob"
+
     def test_accept_wildcard_specificity(self):
         accept = self.storage_class([('asterisk', 0), ('star', 0.5), ('*', 1)])
         assert accept.best_match(['star', 'asterisk'], default=None) == 'star'
@@ -921,9 +1001,40 @@ class TestAccept(object):
         assert accept.best_match(['asterisk'], default=None) is None
 
 
+class TestMIMEAccept(object):
+    storage_class = datastructures.MIMEAccept
+
+    def test_accept_wildcard_subtype(self):
+        accept = self.storage_class([('text/*', 1)])
+        assert accept.best_match(['text/html'], default=None) == 'text/html'
+        assert accept.best_match(['image/png', 'text/plain']) == 'text/plain'
+        assert accept.best_match(['image/png'], default=None) is None
+
+    def test_accept_wildcard_specificity(self):
+        accept = self.storage_class([('*/*', 1), ('text/html', 1)])
+        assert accept.best_match(['image/png', 'text/html']) == 'text/html'
+        assert accept.best_match(['image/png', 'text/plain']) == 'image/png'
+        accept = self.storage_class([('*/*', 1), ('text/html', 1),
+                                     ('image/*', 1)])
+        assert accept.best_match(['image/png', 'text/html']) == 'text/html'
+        assert accept.best_match(['text/plain', 'image/png']) == 'image/png'
+
+
 class TestFileStorage(object):
     storage_class = datastructures.FileStorage
 
     def test_mimetype_always_lowercase(self):
         file_storage = self.storage_class(content_type='APPLICATION/JSON')
         assert file_storage.mimetype == 'application/json'
+
+    def test_bytes_proper_sentinel(self):
+        # ensure we iterate over new lines and don't enter into an infinite loop
+        import io
+        unicode_storage = self.storage_class(io.StringIO(u"one\ntwo"))
+        for idx, line in enumerate(unicode_storage):
+            assert idx < 2
+        assert idx == 1
+        binary_storage = self.storage_class(io.BytesIO(b"one\ntwo"))
+        for idx, line in enumerate(binary_storage):
+            assert idx < 2
+        assert idx == 1
